#include <sdl/sdl.h>

#pragma comment(lib,"sdl.lib")
#pragma comment(lib,"sdlmain.lib")

#define XRESOL 1000
#define YRESOL 800
#define FULLSCREEN 0
#define NBSPRITE 2000

struct Sprite
{
	int x,y;
	int vx,vy;
	SDL_Surface* srf;
};

SDL_Surface* Charger(const char* fic)
{
	SDL_Surface *res;
	SDL_Surface* tmp = SDL_LoadBMP(fic);
	if (tmp==NULL)
	{
		printf("Erreur chargement %s\n",fic);
		exit(-1);
	}
	res = SDL_DisplayFormat(tmp);
	SDL_FreeSurface(tmp);
	return res;
}

int KeyHit()
{
	SDL_Event e;
	if (SDL_PollEvent(&e))
		if (e.type == SDL_KEYDOWN)
			return 1;
	return 0;
}

void Blit(SDL_Surface* source,SDL_Surface* dest,int x,int y)
{
	SDL_Rect R;
	R.x = x;
	R.y = y;
	SDL_BlitSurface(source,NULL,dest,&R);
}

void Initialiser(struct Sprite* sp,SDL_Surface* srf)
{
	sp->x = rand()%(XRESOL-50);
	sp->y = rand()%(YRESOL-50);
	sp->vx = rand()%7-3;
	sp->vy = rand()%7-3;
	if (sp->vx==0 && sp->vy==0)
		sp->vx = 1;
	sp->srf = srf;
}

void Evoluer(struct Sprite* sp)
{
	sp->x+=sp->vx;
	sp->y+=sp->vy;
	if (sp->vx<0 && sp->x<0)
		sp->vx = -sp->vx;
	if (sp->vy<0 && sp->y<0)
		sp->vy = -sp->vy;
	if (sp->vx>0 && sp->x>(XRESOL-50))
		sp->vx = -sp->vx;
	if (sp->vy>0 && sp->y>(YRESOL-50))
		sp->vy = -sp->vy;
}

int main(int argc,char** argv)
{
	struct Sprite TabSprite[NBSPRITE];
	SDL_Surface* balle,*screen;
	int i;
	SDL_Init(SDL_INIT_VIDEO);
	if (FULLSCREEN)
		screen=SDL_SetVideoMode(XRESOL,YRESOL,32,SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);  
	else
		screen=SDL_SetVideoMode(XRESOL,YRESOL,32,SDL_SWSURFACE|SDL_DOUBLEBUF);  
	balle=Charger("balle.bmp");
	SDL_SetColorKey(balle,SDL_SRCCOLORKEY ,SDL_MapRGBA(balle->format,0,0,0,0));
	for(i=0;i<NBSPRITE;i++)
	{
		Initialiser(&TabSprite[i],balle);
	}
	while(!KeyHit())
	{
		SDL_FillRect(screen,NULL,0);
		for(i=0;i<NBSPRITE;i++)
		{
			Evoluer(&TabSprite[i]);
			Blit(TabSprite[i].srf,screen,TabSprite[i].x,TabSprite[i].y);
		}
		SDL_Flip(screen);
	}
	SDL_FreeSurface(balle);
	SDL_Quit();
	return 0;
}


/*!F
	Pour faire bouger plusieurs objets en mme temps, certains prconisent les threads.
	Je pense que c'est beaucoup trop violent.

	Voici comment le faire, sans threads.

	Tout d'abord, considrons une structure Sprite. Un sprite est un objet mobile.
	Dedans, coordonnes x,y et vecteur de dplacement vx,vy : dans quel sens part le sprite.

	Dans le main, un tableau de Sprites (de taille NBSPRITE, du #define plus haut).
	Amusez vous  lancer le programme, puis modifiez les #define en haut pour voir :) 
	Vous pouvez galement changer la rsolution d'cran et le fait d'tre en plein cran ou non.

	Je charge une balle comme dans les exemples prcdents. 
	Puis je fais un for, et je lance Initialiser sur chaque sprite.

	Regardons cette fonction : je dfinis une position x,y alatoirement pour chaque sprite.
	Je dfinis que chaque image est une balle (pour chacun)
	Je dfinis un vecteur de dplacement : rand()%7-3 va nous sortir un nombre alatoire entre -3 et 3
	Si x et y valent 0, je dis que x = 1, car je ne veux pas de balle qui ne bouge pas.

	Retournons dans le main, nous entrons dans le while (qui se coupera si vous appuyez sur une touche)
	Dedans, je repeins le fond en noir, puis je fais un for sur chaque sprite :

	- il volue
	- je l'affiche

	Pour l'volution, j'additionne vx  x et vy  y : dplacement selon le vecteur.
	Puis, pour grer les rebonds, 4 if : si je suis trop  gauche et que le vecteur va  gauche, on le retourne pour qu'il
	reparte vers la droite. Pareil pour la droite, haut, bas.
	Voila donc comment je fais bouger mon sprite : en changeant x,y dans la structure.

	Pour l'affichage, c'est la fonction blit que j'ai employ dans l'exemple prcdent. Je lui passe l'image du sprite,
	le x et le y.

	Et voila.

	Amusez vu  changer l'entier du #define du haut pour voir :)

	La fonction Charger : 
	---------------------

	Je charge l'image comme avant, je sors en mettant un message si je ne la trouve pas.
	Ce qui est intressant est le SDL_DisplayFormat, ou je recre une deuxime surface  partir de la premire, qui aura comme particularit d'avoir
	les mmes proprits que le "vido buffer", c'est  dire votre "screen".
	Si vous ne le faites pas, et que les surfaces n'ont pas le mme format, elles sont converties  chaque blit : c'est long et lourd.
	Si vous le faites, vous gagnez normment en vitesse. 
	Pour vous en convaincre, essayez de virer le SDL_Delay  la fin du main, et lancez le programme : la surface dfilera trs vite. 
	Shuntez le SDL_DisplayFormat en retournant tmp juste avant, vous verrez que ce sera beaucoup plus lent.

!*/

/*!E
	No explanations yet.
!*/

/*
titre: Dplacer plusieurs objets en mme temps.
titreE: Moving multiple objets simultaneously.
meta: C;SDL;en mme temps
metaE: C;SDL;same time moving
desc : Dplacer plusieurs objets.
descE : Moving objets simultaneously.
cle  : 
linkA: balle.bmp
parag: 2.2. Concept de sprites.
paragE: 2.2. Sprite concept.
*/
